package com.bergerkiller.bukkit.nolagg;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;

import org.bukkit.Material;
import org.bukkit.plugin.Plugin;

import com.all.bukkit.threadcheck.ThreadCheck;
import com.bergerkiller.bukkit.common.config.ConfigurationNode;
import com.bergerkiller.bukkit.common.config.FileConfiguration;
import com.bergerkiller.bukkit.common.Task;
import com.bergerkiller.bukkit.common.utils.EnumUtil;
import com.bergerkiller.bukkit.nolagg.base.NoLaggBase;
import com.bergerkiller.bukkit.nolagg.meta.ChunkMetaData;
import com.bergerkiller.bukkit.nolagg.meta.WorldMetaData;
import com.bergerkiller.bukkit.nolagg.spawnlimit.SpawnHandler;

public class NoLagg extends NoLaggBase {

	public NoLagg() {
		super(2002, 2032);
	}

	public static NoLagg plugin;

	private Task updateTask;
	public int updateInterval;

	public static boolean useSpawnLimits;
	public static boolean isSpoutEnabled = false;
	public static boolean autoFixLighting = true;
	public static boolean stackOrbs = true;
	public static boolean ignoreMobLimits = false;
	public static boolean isChunkEnabled = false;
	public static boolean isTNTEnabled = false;
	public static boolean isLightingEnabled = false;

	public static Map<String, List<String>> clearShortcuts = new HashMap<String, List<String>>();

	@Override
	public void updateDependency(Plugin plugin, String pluginName, boolean enabled) {
		if (pluginName.equals("Spout")) {
			isSpoutEnabled = enabled;
		} else if (pluginName.equals("NoLaggChunks")) {
			isChunkEnabled = enabled;
		} else if (pluginName.equals("NoLaggTNT")) {
			isTNTEnabled = enabled;
		} else if (pluginName.equals("NoLaggLighting")) {
			isLightingEnabled = enabled;
		} else if (pluginName.equals("EntitySuppresor")) {
			if (ignoreMobLimits = enabled) {
				log(Level.WARNING, "Entity suppresor detected: Mobs will not be limited!");
			}
		}
	}

	public void enable() {		
		ThreadCheck.init(this);
		plugin = this;

		//General registering
		this.register(NLListener.class);
		this.registerNextTick(NLMonitoredListener.class);
		this.register(new Commands());

		this.loadConfig();
		StackFormer.init();
		this.updateTask = new Task(this) {
			public void run() {
				try {				
					WorldMetaData.updateAll();
				} catch (Throwable t) {
					log(Level.SEVERE, "An error occured while performing a routine update:");
					t.printStackTrace();
				}
			}
		}.start(0, updateInterval);

		AutoSaveChanger.init();
	}
	public void disable() {
		Task.stop(this.updateTask);
		PerformanceMonitor.deinit();
		SpawnHandler.clear();
		AutoSaveChanger.deinit();
		WorldMetaData.deinit();
		StackFormer.deinit();
		ThreadCheck.deinit();
	}

	public void loadConfig() {
		//General settings
		FileConfiguration config = new FileConfiguration(this);
		boolean regen = !config.exists();
		config.load();

		//Top header
		config.setHeader("This is the configuration of NoLagg, in here you can enable or disable features as you please");
		config.addHeader("For more information, you can visit the following websites:");
		config.addHeader("http://dev.bukkit.org/server-mods/nolagg/");
		config.addHeader("http://forums.bukkit.org/threads/nolagg.36986/");

		//unload delay
		config.setHeader("chunkUnloadDelay", "");
		config.setHeader("chunkUnloadDelay.enabled", "Whether or not chunks are unloaded after a set delay (bugged in R4-5!)");
		config.setHeader("chunkUnloadDelay.delay", "The unload delay in milliseconds");
		ChunkMetaData.useUnloadDelay = config.get("chunkUnloadDelay.enabled", false);
		ChunkMetaData.unloadDelay = config.get("chunkUnloadDelay.delay", 10000);

		//item stacking
		config.setHeader("itemStacking", "");
		config.setHeader("itemStacking.enabled", "Whether or not items are stacked to one item");
		config.setHeader("itemStacking.radius", "The block radius to look for other items when stacking");
		config.setHeader("itemStacking.threshold", "The amount of (physical) items needed to form one stack");
		ItemHandler.formStacks = config.get("itemStacking.enabled", true);
		StackFormer.stackRadiusSquared = Math.pow(config.get("itemStacking.radius", 1.0), 2);
		StackFormer.stackThreshold = config.get("itemStacking.threshold", 2);

		//ignored types
		if (regen) {
			config.set("ignoredItemTypes", Arrays.asList("DIAMOND_PICKAXE", "WOODEN_HOE"));
		}
		config.setHeader("ignoredItemTypes", "");
		config.addHeader("ignoredItemTypes", "The item types (materials) to ignore during item stacking, buffering and spawn limits");
		config.addHeader("ignoredItemTypes", "Use 'orb' to ignore experience orbs");
		ItemHandler.clearIgnoredTypes();
		stackOrbs = true;
		for (String type : config.getList("ignoredItemTypes", String.class)) {
			if (type.equalsIgnoreCase("experienceorb") || type.equalsIgnoreCase("xporb") || type.equalsIgnoreCase("orb")) {
				stackOrbs = false;
				continue;
			}
			Material mat = EnumUtil.parseMaterial(type, null);
			if (mat != null) {
				ItemHandler.addIgnoredType(mat);
			} else {
				log(Level.WARNING, "Unknown item type found in configuration: " + type);
			}
		}

		//general intervals and settings
		config.setHeader("autoSaveInterval", "");
		config.addHeader("autoSaveInterval", "The tick interval at which the server saves automatically (20 ticks = 1 second)");
		config.setHeader("updateInterval", "The tick interval at which NoLagg updates internally (item stacking, buffering and chunk unloading) (20 ticks = 1 second)");
		config.setHeader("monitorInterval", "The tick interval at which performance information is sent and/or logged (20 ticks = 1 second)");
		config.setHeader("startLoggingOnStartup", "Whether or not to start logging to the log.txt file when NoLagg enables");
		config.setHeader("autoFixLighting", "Whether or not the lighting of newly-generated chunks is fixed");
		AutoSaveChanger.newInterval = config.get("autoSaveInterval", 400);
		updateInterval = config.get("updateInterval", 20);
		PerformanceMonitor.deinit();
		PerformanceMonitor.monitorInterval = config.get("monitorInterval", 40);
		PerformanceMonitor.sendLog = config.get("startLoggingOnStartup", false);
		PerformanceMonitor.init();
		autoFixLighting = config.get("autoFixLighting", true);

		//clear shortcuts
		if (regen) {
			ConfigurationNode node = config.getNode("clearShortcuts");
			node.set("enemies", Arrays.asList("monster"));
			node.set("notneutral", Arrays.asList("monster", "item", "tnt", "egg", "arrow"));
		}
		ConfigurationNode shortc = config.getNode("clearShortcuts");
		shortc.setHeader("");
		shortc.addHeader("Several shortcuts you can use for the /nolagg clear(all) command");
		for (String key : shortc.getKeys()) {
			clearShortcuts.put(key.toLowerCase(), shortc.getList(key, String.class));
		}

		//spawn limits
		config.setHeader("useEntitySpawnLimits", "");
		config.addHeader("useEntitySpawnLimits", "Whether or not entity spawn limits are used");
		useSpawnLimits = config.get("useEntitySpawnLimits", true);

		if (regen) {
			ConfigurationNode limits = config.getNode("spawnlimits");
			ConfigurationNode node = limits.getNode("default");
			node.set("mob", 800);
			node.set("cow", 60);
			node.set("item", 600);
			node.set("creepers", 30);
			node.set("monsters", 400);
			node = limits.getNode("worlds");
			node.set("world1.monsters", 300);
			node.set("world2.chickens", 30);
			node = limits.getNode("global");
			node.set("mobs", 3000);
			limits = config.getNode("mobSpawnerLimits");
			limits.set("default.mob", 300);
			limits.set("default.zombie", 30);
			limits.set("worlds.world3.cavespider", 0);
			limits.set("worlds.creativeworld.mob", 0);
			limits.set("global.mob", 1000);
			//others
		}

		config.setHeader("spawnlimits", "");
		config.addHeader("spawnlimits", "The general spawn limits (natural spawning)");
		config.addHeader("mobSpawnlimits", "For more information, see http://dev.bukkit.org/server-mods/nolagg/pages/spawn-limits-nolagg/");
		config.setHeader("spawnlimits.default", "The default spawn limits per world, overridden by world limits");
		config.setHeader("spawnlimits.worlds", "The world-specific spawn limits");
		config.setHeader("spawnlimits.global", "The global spawn limits");

		config.setHeader("mobSpawnerLimits", "");
		config.addHeader("mobSpawnerLimits", "The spawn limits for mob spawners");
		config.addHeader("mobSpawnerLimits", "For more information, see http://dev.bukkit.org/server-mods/nolagg/pages/spawn-limits-nolagg/");
		config.setHeader("mobSpawnerLimits.default", "The default spawn limits per world, overridden by world limits");
		config.setHeader("mobSpawnerLimits.worlds", "The world-specific spawn limits");
		config.setHeader("mobSpawnerLimits.global", "The global spawn limits");

		SpawnHandler.clear();
		if (useSpawnLimits) {
			//Spawn restrictions
			SpawnHandler.GENERALHANDLER.load(config.getNode("spawnlimits"));
			//Mob spawn restrictions
			SpawnHandler.MOBSPAWNERHANDLER.load(config.getNode("mobSpawnerLimits"));
		}

		//Write out data
		config.save(); 
	}

	@Override
	public void permissions() {
		this.loadPermissions(Permission.class);
	}

}
